-- clipping reagon - the overlap of two polygons
-- Sutherland-Hodgman_polygon_clipping
-- Caution: Fill should only work for convex polygons!!!

local sin, cos, atan2, sqrt, pi, floor = math.sin, math.cos, math.atan2, math.sqrt, math.pi, math.floor
local random, max, min = math.random, math.max, math.min
local gr, tm, ky = love.graphics, love.timer, love.keyboard
local v = require 'vector'

function love.load()
	gr.setCaption('Clipping (overlapping area of two polygons)')
	gr.setMode( 400, 400, {} )
	fig1 = {
		sides	= 4,
		radius	= 150,
		center	= {205,200}
		}
	obj1 = createObject(fig1)
	fig2 = {
		sides	= 16,
		radius	= 75,
		center	= {200,200}
		}
	obj2 = createObject(fig2)

	poly_center(obj2,{300,175})
	overlap = clip( obj1, obj2 )
	dir = 1		-- movement direction (1 = right)
	end

function love.draw()
	gr.setColor( 0, 0, 255, 255 )
	gr.polygon( 'line', convert_to_serial( obj1 ) )
	gr.print( 'host polygon', 10, 320 )
	gr.setColor( 255, 0, 0, 255 )
	gr.polygon( 'line', convert_to_serial( obj2 ) )
	gr.print( 'clip polygon', 10, 340 )
	gr.setColor( 0, 255, 0, 155 )
	if #overlap > 3 then
		gr.polygon( 'fill', convert_to_serial( overlap ) )
		else dir = -dir
		end
	gr.print( 'clipping area', 10, 360 )
	gr.setColor( 255, 255, 0, 255 )
	show_normals( overlap )
	gr.print( 'surface normals', 10, 380 )
	gr.setColor( 255, 0, 255, 255 )
	display_with_offset( overlap,{ 100, -50 } )
	love.graphics.print( 'fps: '..love.timer.getFPS(), 0, 0 )
	end

function show_normals( p )
	p[0] = p[#p]
	for i = 0, #p - 1 do
		av		= v.mid( p[i], p[i+1] )
		norm	= v.edge_normal( p[i], p[i+1] )
		gr.line( av[1], av[2], norm[1], norm[2] )
		gr.print(i,p[i][1],p[i][2])
		end
	end

function love.update( dt )
	for i = 1, #obj2 do
		obj2[i][1] = obj2[i][1] + dir * 50 * dt
		end
	overlap = clip( obj1, obj2 )
	-- correct for non-ccw rotation - (not necessary)
	--if areaPolygon(overlap) < 0  then overlap = reversePointOrder(overlap) end
	end

function love.keypressed( key )
	if key == 'escape'	then love.event.push( 'quit' )	end
	end

------------------------------------
--          clipping code         --
--  SutherlandHodgman algorithm  --
--  modification-Rosettacode.org  --
--    (based on line intersect)   --
------------------------------------

function inside(p, lp1, lp2)	-- see if point on line segment
	return (lp2[1]-lp1[1])*(p[2]-lp1[2]) >= (lp2[2]-lp1[2])*(p[1]-lp1[1]) -- = added?
	end

function clip( src, cut )
	local overlap		= src			-- walking around clip points - line segmant = cp1,cp2
	local cp1			= cut[ #cut ]	-- clip point one (last point)
	for j = 1, #cut do
		local cp2		= cut[j]		-- clip point two (one up from cp1)
		local List		= overlap		-- save points in point List
		overlap			= {}			-- clear out everything in overlap
		local s 		= List[ #List ]	-- walk down List - line s,e (first round = all src points)
		for i = 1, #List do
			local e		= List[i]
			if inside(e, cp1, cp2) then				-- two polygons in contact
				if not inside( s, cp1, cp2 ) then	-- point e not inside clipping area
					overlap[ #overlap+1 ] = v.intersection( cp1, cp2, s, e ) -- add contact point
					end
				overlap[ #overlap+1 ] = e			-- add last point to List
				elseif inside( s, cp1, cp2 ) then
				overlap[ #overlap+1 ] = v.intersection( cp1, cp2, s, e )
				end
			s = e
			end
		cp1 = cp2
		end
	return overlap
	end
-------------------------
function convert_to_serial(t)-- reformat data (paired => serial)
	local r = {}
	for i = 1, #t do
		r[#r+1] = t[i][1]
		r[#r+1] = t[i][2]
		end
	return r
	end

function createObject( d )
	local tab = {}
	for i = 0, 2 * pi, 2 * pi / d.sides do
		tab[#tab+1] = {
			d.radius * cos(i) + d.center[1],
			d.radius * sin(i) + d.center[2] }
		end
	return tab
	end

function display_with_offset( p, of )
	if #p>=3 then
		local tab={}
		for i=1,#p do
			tab[i]={p[i][1]/2+of[1],p[i][2]/2+of[2]}
			end
		gr.polygon('fill',convert_to_serial(tab))
		end
	end

function areaPolygon( p )
	if #p < 3 then return 0 end			-- need at least 3 points to define area
	local area = 0
	for i=1,#p do
		area = area + v.det( p[i], p[i%(#p)+1])
		end
	p.area = area/2
	end

function reversePointOrder(p)
	local tmp = {}						-- reverse data point order (cw => ccw or reverse)
	for i = #p, 1, -1 do
		tmp[ #tmp + 1 ] = p[i]
		end
	return tmp
	end

-- ================================================= --
function poly_findCenter( p )
	-- locate center & calculate area of polygon
	local cent1, cent2 = 0, 0
	for i = 1,#p do
		det		= v.det( p[i], p[i%(#p)+1] )
		cent1	= cent1 + ( p[i%(#p)+1][1] + p[i][1] ) * det
		cent2	= cent2 + ( p[i%(#p)+1][2] + p[i][2] ) * det
		end
	poly_area( p )
	p.cent	= {cent1 / (6 * p.area), cent2 / (6 *p.area)}
	end
function poly_area( p )
	if #p < 3 then return 0 end			-- need at least 3 points to define area
	local area = 0
	for i=1,#p do
		area = area + v.det( p[i], p[i%(#p)+1])
		end
	p.area = area/2
	end
function poly_center( p, cent ) -- moves polygon center to specifed location
	if not p.cent then poly_findCenter( p ) end
	poly_translate( p, v.sub( cent, p.cent ) )
	p.cent = {cent[1], cent[2]}
	end
function poly_unpack( p )
	-- convert {{x1,y1},{x2,y2}...} into {x1,y1,x2,y2, ...} (for use with polygon)
	local tab = {}
	for i = 1, #p do
		tab[#tab+1] = p[i][1]
		tab[#tab+1] = p[i][2]
		end
	return tab
	end
function poly_translate(p, trans ) -- translates polygon points
	for i = 1, #p do
		p[i] = {p[i][1] + trans[1], p[i][2] + trans[2]}
		end
	end
