local gr,sqrt,pi,sin,cos = love.graphics, math.sqrt, math.pi,math.sin,math.cos
local atan2, floor = math.atan2,math.floor

local vector = {}

function vector.det( p1, p2 )
	return p1[1] * p2[2] - p1[2] * p2[1]
	end
function vector.len( p )
	return sqrt(p[1]*p[1] + p[2]*p[2])
	end
function dist(p1, p2)
	return vector.len({p1[1]-p2[1], p1[2]-p2[2]})
	end
function vector.add(p1, p2)
	return p1[1]+p2[1], p1[2]+p2[2]
	end
function vector.sub(p1, p2)
	return {p1[1]-p2[1], p1[2]-p2[2]}
	end
function vector.rot(an, p) -- returns rotated x & y values
	local c, s = cos(an), sin(an)
	return {c*p[1] - s*p[2], s*p[1] + c*p[2]}
	end
function vector.dot(p1, p2)
	return p1[1]*p2[1] +  p1[2]*p2[2]
	end
function vector.dotprod( p1, p2 )	-- returns a scaler
	local sum = 0
	for i = 1, #p1 do
		sum = sum + p1[i] * p2[i]
		end
	return sum
	end
function vector.normalize(p)
	local l = len(p)
	return {p[1]/l, p[2]/l}
	end
function vector.angle( p1, p2 )
	-- angle of line (slope) in degrees
	return 180 * atan2( (p2[2]-p1[2]),(p2[1]-p1[1]) ) / pi
	end
function vector.slope(p1,p2)
	return ( p2[2] - p1[2] ) / ( p2[1] - p1[1] )
	end
function vector.equat( p1, p2 )
	-- returns equation for line connecting two points
	local slope = ( p2[2] - p1[2] ) / ( p2[1] - p1[1] )
	local inter = p2[2] - slope * p2[1]
	return slope, inter
	end
function vector.ccw(p, q, r)
	return vector.det({q[1]-p[1], q[2]-p[2]}, {r[1]-p[1], r[2]-p[2]}) >= 0
	end
function vector.str(p)
	return "("..tonumber(p[1])..","..tonumber(p[2])..")"
	end
function vector.perpendicular(p)
	return -p[2],p[1]
	end
function vector.mid( p1, p2 )
	return { ( p1[1] + p2[1] ) / 2, ( p1[2] + p2[2] ) / 2 }
	end
function vector.edge_normal( p1, p2 )
	local av	= vector.mid( p1, p2 )
	local t90	= -pi/2
	local nx	= p2[1] - av[1]
	local ny	= p2[2] - av[2]
	local x 	= nx * cos( t90 ) - ny * sin( t90 ) + av[1]
	local y 	= nx * sin( t90 ) + ny * cos( t90 ) + av[2]
	return { x, y }
	end
function vector.intersection( p1s, p1e, p2s, p2e)
-- intersection of two lines defined by p1s,p1e & p2s,p2e
-- not constrained to limits of line segments
	local dc	= vector.sub( p1s,	p1e )
	local dp 	= vector.sub( p2s,	p2e	)
	local n1	= vector.det( p1s,	p1e	)
	local n2	= vector.det( p2s,	p2e)
	local n3	= 1 / vector.det( dc,	dp	)
	local x		= (n1 * dp[1] - n2 * dc[1]) * n3
	local y		= (n1 * dp[2] - n2 * dc[2]) * n3
	return { x, y }
	end
function vector.onLine( p,lp1, lp2 )
-- whether point lies of line segment defined by points lp1 & lp2
	local sl, int = vector.equat( lp1, lp2 )
	local testx, testy
	if p[2] == sl * p[1] + int then
		if lp2[1] > lp1[1] then
			if p[1] >= lp1[1] and p[1] <= lp2[1] then testx = true end
			else
			if p[1] >= lp2[1] and p[1] <= lp1[1] then testx = true end
			end
		if lp2[2] > lp1[2] then
			if p[2] >= lp1[2] and p[2] <= lp2[2] then testy = true end
			else
			if p[2] >= lp2[2] and p[2] <= lp1[2] then testy = true end
			end
		end
	return  testx and testy
	end

return vector
